% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : Annalyse spectrale d'un segment par ARWARP
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%       warping
% --------------------
% En entree :
%               x : Signal 
%               P : model order
%               b : warping parameter
%               freq_norm : vecteur des freq normalisees
% --------------------
% En sortie :
%               SpectreSLA : ARWARP spectrum
% --------------------
function [SpectreSLA]= calculARWARP(x,P,b,freq_norm)

% fonction deformation de l'echelle frequentielle
% correspondant au frequency warping
func_theta=@(b,w) w+2*atan(b*sin(w)./(1-b*cos(w))); % equation (2.5), rapport tache 3, page 7
func_theta_diff=@(b,w) 1 - (2*((b^2*sin(w)^2)/(b*cos(w) - 1)^2 + (b*cos(w))/(b*cos(w) - 1)))/((b^2*sin(w)^2)/(b*cos(w) - 1)^2 + 1);

%Compensation du filtre Lambda0:
Alambda0=[1 -b];
puls=2*pi*freq_norm; %entre 0 et pi
Hlambda0=freqz(sqrt(1-b^2),Alambda0,puls); % filtre Lambda0

N = length(x);          % Number of samples in input signal           
freq_warp=func_theta(b,2*pi*freq_norm')/(2*pi); % echelle des frequences warpee
                                 
% nombre de coefs warpes a prendre en compte
    NFilters = round(N/func_theta_diff(b,pi));   % Nombre de filtres

xwarp = warping(x,NFilters,b)'; % frequency warping

M = length(xwarp); % taille de la fenetre = longueur des segments du signal
A = toeplitz(xwarp(P:M-1),xwarp(P:-1:1));
aest = -A\xwarp(P+1:M);    
EPLwarp = A*aest + xwarp(P+1:M); %LPE
      
%%%%%%%% Compute sigma
EPL = warping(EPLwarp,N,-b); %unwarping the LPE
sigma2estfinal = var(EPL);


freqAR = exp(1i*2*pi*(0:P)'*freq_warp');
SpectreSLA = sigma2estfinal ./ (abs([1;aest]'*freqAR).^2); 
SpectreSLA = SpectreSLA.*abs(Hlambda0).^2;    % Multiply by the filter


end

