% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : fonction d'estimation de la pente et de l'ecart-type du bruit
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%      get_index_norm
% --------------------
% En entree :
    % SpectreSLA : spectre a analyser
    % freq : vecteur des frequences associees en km-1
    % Fe : frequence d'echantillonnage en km-1
    % freqpente : bornes de calcul de la pente en km-1
    % freqplateaubruit : bornes de calcul du plateau de bruit en km-1
% --------------------
% En sortie :
    % pentespectre : pente estimee
    % sigmabruit : sigma du bruit
        % voir mail Amandine 20/10 : sigmabruit = sqrt (K/(2*dist)) K etant le niveau du
        % plancher lu sur le spectre
    % Sth : spectre theorique modele qui sur une representation loglog donne :
        % ----Smax
        %      \
        %       \
        %        Fcross-----K = plateaubruit
    % Fcross : frequence de croisement entre plateau de bruit et pente (en km-1)
% --------------------

function [pentespectre, sigmabruit,Sth] = estimePenteBruit(SpectreSLA,freq_norm,freqpente_norm,freqplateaubruit_norm)
% parametres pour calculs pente et bruit
% bornes de calcul en frequences normalisees
f_norm_begin_noise = freqplateaubruit_norm(1); %normalized frequency
f_norm_end_noise = freqplateaubruit_norm(2); %

tab_log_f = log(freq_norm); % il faut utiliser la meme echelle frequentielle que le spectre
% pour calculer la pente
ind_lin_reg = get_index_norm(freqpente_norm,tab_log_f);

% on ecrit : log(Spectre) = coef(1) + coef(2)* log(frequence)
% soit pour plusieurs valeurs de frequences :
% log(S) = (1 1 ... 1 log(f1) log(f2)... log(fn) ) x [coef(1) coef(2)]'
% soit log(S) = H x coef_linreg
% et donc coef_linreg = H \ log(S) soit inv(H)*log(S)
H = [ones(length(ind_lin_reg),1) tab_log_f(ind_lin_reg)'];
coef_linreg = H\(log(SpectreSLA(ind_lin_reg)))'; % vecteur des coefs de la regression lineaire
S_linreg = H * coef_linreg;
pentespectre = -coef_linreg(2);

% pour calculer le niveau de bruit
ind_lin_reg_noise = get_index_norm(freqplateaubruit_norm,tab_log_f);

%%% calcul ecart-type de bruit
sigmabruit = mean(SpectreSLA(ind_lin_reg_noise));
ind_begin_slope = ind_lin_reg(1);
% calcul du spectre modele
% log(Sth) : constant au debut jusqu'au debut de la pente, 
% puis pente (entre log(Sth) et log(freq)
% puis plateau de bruit
plateaubruit = mean(SpectreSLA(ind_lin_reg_noise)); % valeur du plateau de bruit sur le spectre en lineaire
Sth = ones(size(SpectreSLA)) * plateaubruit; % on met deja cette valeur 
% on corrige le debut
Sth(1 : ind_begin_slope) = ones(1,ind_begin_slope)*exp(S_linreg(1)); % plateau du debut (Smax)
% (S_linreg est calcule sur log(SpectreSLA) )
% on corrige la pente
log_Fcross = (log(plateaubruit)-coef_linreg(1)) / coef_linreg(2); % log de la frequence de croisement normalisee
Fcross = exp(log_Fcross); % frequence de croisement
ind_fcross = find(freq_norm >= Fcross,1,'first'); % indice de la frequence de croisement
Sth(ind_begin_slope : ind_fcross) = exp(coef_linreg(1) + coef_linreg(2)*log(freq_norm(ind_begin_slope : ind_fcross)));
    